/*

Animator.js

Copyright (c) 2006 Scue

www.scue.co.uk (www.scue.co.uk/widgets/)
info@scue.co.uk
developer: aidan kane (aidan@scue.co.uk)

see LICENSE for details of the license

This file contains code derived from the Apple librarys

*/


// Animator class to perform time based animations

function Animator()
{
	this.starttime = 0;
	this.duration = 400;
	this.timer = null;
	this.from = 0;
	this.to = 0;
	this.now = 0;

	// element being animated
	this.element = null;

	// function to call the at each step of animation
	this.animationStep = null;
	// function to call when animation finished
	this.whenfinished = null;
}


// implementation details
// ----------------------

Animator.prototype = 
{

	animate : function()
	{
		var T;
		var ease;
		var time = (new Date).getTime();
		var finishedanimation = false;

		T = this.limit_3(time-this.starttime, 0, this.duration);

		if (T >= this.duration)
		{
			clearInterval(this.timer);
			this.timer = null;
			this.now = this.to;
			finishedanimation = true;
		}
		else
		{
			ease = 0.5 - (0.5 * Math.cos(Math.PI * T / this.duration));
			this.now = this.computeNextFloat(this.from, this.to, ease);
		}

		// user defined function
		this.animationStep(this.element, this.now);

		if (finishedanimation)
		{
			// call the user's post animation function
			if (this.whenfinished != null)
			{
				setTimeout(this.whenfinished,0);
			}
		}
	},

	startAnimation : function()
	{
		var starttime = (new Date).getTime() - 13;
		this.starttime = starttime;

		// run the animation
		var localThis = this;
		this.timer = setInterval( function () {localThis.animate();}, 13);
		this.animate();
	},

	pauseAnimation : function()
	{
		// stop the clock!
		if (this.timer != null)
		{
			clearInterval(this.timer);
			this.timer = null;
		}
	},

	limit_3 : function(a, b, c)
	{
		return a < b ? b : (a > c ? c : a);	
	},

	computeNextFloat : function(from, to, ease)
	{
		return from + (to - from) * ease;
	}
}
